SafeDropGraph

Struct SafeDropGraph 

pub struct SafeDropGraph<'tcx> {
    pub alias_graph: AliasGraph<'tcx>,
    pub bug_records: BugRecords,
    pub drop_record: Vec<DropRecord>,
    pub adt_owner: OHAResultMap,
}
We represent each target function with the SafeDropGraph struct and then perform analysis based on the struct.

Fields

alias_graph: AliasGraph<'tcx>bug_records: BugRecordsdrop_record: Vec<DropRecord>adt_owner: OHAResultMap

Implementations

Source

impl<'tcx> SafeDropGraph<'tcx>

pub fn alias_bb(&mut self, bb_index: usize)

pub fn alias_bbcall(&mut self, bb_index: usize, fn_map: &MopFnAliasMap)

Source

impl<'tcx> SafeDropGraph<'tcx>

pub fn add_to_drop_record( &mut self, value_idx: usize, bb_idx: usize, flag_cleanup: bool, )

pub fn push_drop_info(&mut self, value_idx: usize, drop_spot: LocalSpot)

pub fn push_drop_top_down(&mut self, value_idx: usize, drop_spot: LocalSpot)

drop the fields of the root node.

pub fn push_drop_bottom_up(&mut self, value_idx: usize, drop_spot: LocalSpot)

pub fn fetch_drop_info(&mut self, value_idx: usize)

pub fn clear_drop_info(&mut self, value_idx: usize)

pub fn clear_father_drop(&mut self, value_idx: usize)

pub fn clear_field_drop(&mut self, value_idx: usize)

pub fn fetch_drop_from_bottom(&mut self, value_idx: usize)

pub fn fetch_drop_from_top(&mut self, value_idx: usize)

pub fn fetch_drop_from_alias(&mut self, value_idx: usize)

Source

impl<'tcx> SafeDropGraph<'tcx>

pub fn new(tcx: TyCtxt<'tcx>, def_id: DefId, adt_owner: OHAResultMap) -> Self

pub fn from_path_graph( tcx: TyCtxt<'tcx>, def_id: DefId, path_graph: PathGraph<'tcx>, adt_owner: OHAResultMap, ) -> Self

pub fn sync_drop_record(&mut self)

Ensure drop_record matches the current length of alias_graph.values. Call this after any alias operation that may create new value nodes (projection, sync_field_alias, sync_father_alias, handle_fn_alias, etc.).

Source

impl<'tcx> SafeDropGraph<'tcx>

pub fn drop_check(&mut self, bb_idx: usize)

pub fn drop_heap_item_check(&self, place: &Place<'tcx>) -> bool

pub fn process_function_paths(&mut self, fn_map: &MopFnAliasMap)

Process pre-enumerated whole-function paths for SafeDrop via DFS on the path tree. All paths have already been filtered by incremental constraint-based reachability checks during enumeration, so no per-path filtering is needed. State is saved at branch points and restored before each sibling subtree.

pub fn process_function_paths_opt( &mut self, precomputed_paths: Option<PathTree>, fn_map: &MopFnAliasMap, )

fn dfs_safedrop( &mut self, node: &PathNode, path: &mut Vec<usize>, fn_map: &MopFnAliasMap, ) -> Result<(), ()>

pub fn report_bugs(&self)

fn make_bug( &self, idx: usize, trigger_info: LocalSpot, span: Span, confidence: usize, bug_type: BugType, ) -> TyBug

fn check_drop_status(&mut self, idx: usize) -> Option<usize>

pub fn uaf_check( &mut self, value_idx: usize, bb_idx: usize, span: Span, is_fncall: bool, )

pub fn rate_confidence(kind: ValueKind, fully_dropped: bool) -> usize

pub fn df_check( &mut self, value_idx: usize, bb_idx: usize, span: Span, flag_cleanup: bool, ) -> bool

pub fn dp_check(&mut self, flag_cleanup: bool)

fn dp_check_arg(&mut self, arg_idx: usize, flag_cleanup: bool)